"""Common physical constants in SI units."""

LIGHTSPEED: float = 299792458  # Lichtgeschwindigkeit
ELEMENTARY_CHARGE: float = 1.602176634e-19  # Elementarladung
VACUUM_PERMEABILITY: float = 1.25663706212e-6  # Magnetische Feldkonstante
VACUUM_PERMITTIVITY: float = 8.8541878128e-12  # Elektrische Feldkonstante
COULOMB_CONSTANT: float = 8.9875517922e+9  # Coulomb-Konstante
PLANCK_CONSTANT: float = 6.62607015e-34  # Plancksches Wirkungsquantum
GRAVITATIONAL_CONSTANT: float = 6.67430e-11  # Gravitationskonstante
PLANCK_MASS: float = 2.176434e-8  # Planck Masse
PLANCK_LENGTH: float = 1.616255e-35  # Planck Länge
PLANCK_TIME: float = 5.391247e-44  # Planck Zeit
PLANCK_TEMPERATURE: float = 1.486784e+32  # Planck Temperatur
BOLTZMANN_CONSTANT: float = 1.380649e-23  # Boltzmann Konstante
STEFAN_BOLTZMANN_CONSTANT: float = 5.670374419e-8  # Stefan Boltzmann Konstante
ELECTRON_MASS: float = 9.1093837015e-31  # Elektronenmasse
COMPTON_WAVELENGTH: float = 2.42631023867e-12  # Compton Wellenlänge
ELECTRON_RADIUS: float = 2.8179403262e-15  # klassischer Elektronenradius
RYDBERG_CONSTANT: float = 1.0973731568160e+7  # Rydberg konstante
RYDBERG_FREQUENCY: float = 3.2898419602508e15  # Rydberg Frequenz
AVOGADRO_CONSTANT: float = 6.02214076e23  # Avogadro Konstante
GAS_CONSTANT: float = AVOGADRO_CONSTANT * BOLTZMANN_CONSTANT  # Gaskonstante
# Vm0 = 22.41396954e-3 # Molares Volumen eines idealen Gases
ATOMIC_MASS: float = 1.66053906660e-27  # atomare Masseneinheit
FINE_STRUCTURE_CONSTANT: float = 7.297_352_5693e-3
